function [d, coords] = raycast(x0, y0, theta0, laser_angle, gridworld)

angle = theta0 + laser_angle;

if(gridworld(ceil(x0), ceil(y0)) == 1)
%    'pose is in collision with world, no guarantees about raycasting working'
    d = 0;
    coords = [x0;y0];
    return;
end

if(x0==ceil(x0))
    x0 = x0-0.001;
end
if(y0==ceil(y0))
    y0 = y0-0.001;
end

% not checked for integer x,y --- they should not happen in normal
% conditions

my_eps = 0.0001;
% intersect horizontal lines:


x = x0; y = y0; found = false; 

if( mod(angle,pi)~= pi/2)
    tanangle = tan(angle);
    if (cos(angle) >=0) % x is growing    
        while ( x <= size(gridworld,1)-1 && ~found)
            x = floor(x+1);
            y = y0 + tanangle*(x-x0);
            if (~ ( y <= size(gridworld,2)-1 && y >= 1))
                break;
            end
            if(gridworld(ceil(x+my_eps), ceil(y)))
                xfound_hor = x;
                yfound_hor = y;
                found = 1;
            end
        end
    else
        while ( x >= 1 && ~found)
            x = ceil(x-1);
            y = y0 + tanangle*(x-x0);
            if (~ ( y <= size(gridworld,2)-1 && y >= 1))
                break;
            end
            if(gridworld(ceil(x), ceil(y)))
                xfound_hor = x;
                yfound_hor = y;
                found = 1;
            end
        end
    end
end

if(found)
    found_hor = 1;
else
    found_hor = 0;
end


% intersection vertical lines:

x = x0; y = y0; found = false;

if( mod(angle, pi) ~= 0 )
    cotangle = cot(angle);  
    if(sin(angle) >= 0) % y is growing
        while( y <= size(gridworld, 2) - 1 && ~found )
            y = floor(y+1);
            x = x0 + cotangle*(y-y0);
            if (~ ( x <= size(gridworld,1)-1 && x >= 1))
                break;
            end
            if(gridworld(ceil(x), ceil(y+my_eps)))
                xfound_ver = x;
                yfound_ver = y;
                found = 1;
            end
        end
    else
        while ( y >= 1 && ~ found)
            y = ceil(y-1);
            x = x0 + cotangle*(y-y0);
            if (~ ( x <= size(gridworld,1)-1 && x >= 1))
                break;
            end
            if(gridworld(ceil(x), ceil(y)))
                xfound_ver = x;
                yfound_ver = y;
                found = 1;
            end
        end
    end
end


if(found)
    found_ver = 1;
else
    found_ver = 0;
end

if(~found_ver && ~found_hor)
    'problem!'
end


% check which one was first:

if(found_ver && ~found_hor)
    d_ver = sqrt((xfound_ver-x0)^2 + (yfound_ver-y0)^2);
    d = d_ver;
    coords = [xfound_ver; yfound_ver];
elseif (found_hor && ~found_ver)
    d_hor = sqrt((xfound_hor-x0)^2 + (yfound_hor-y0)^2);
    d = d_hor;
    coords = [xfound_hor; yfound_hor];
else
    d_hor = (xfound_hor-x0)^2 + (yfound_hor-y0)^2; 
    d_ver = (xfound_ver-x0)^2 + (yfound_ver-y0)^2;

    if( d_hor <= d_ver )
        coords(1) = xfound_hor;
        coords(2) = yfound_hor;
        d = sqrt(d_hor);
    else
        coords(1) = xfound_ver;
        coords(2) = yfound_ver;
        d = sqrt(d_ver);
    end
end







